/**************************************************************
 * The SoundExplorer class
 * By: Shaun M. McFall
 * Under: Dr. Krone, Dr. Bressoud
 * Denison University
 * Summer, 2007
 **************************************************************/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.Color;
import javax.sound.sampled.*;

public class SoundExplorer extends JFrame implements KeyListener, MouseListener, MouseMotionListener {
  private int numFrames;
  private Sound snd;
  private int first;
  private int last;
  private int mouseX, mouseY;
  private boolean dragging = false;
  private boolean paint = true;
  private int dragStart;
  private int scale;
  private boolean firstTime = true;
  private boolean playing;
  
  /**
   * Constructs the SoundExplorer using the Sound that
   * is passed in.
   * 
   * @param snd the Sound
   */
  public SoundExplorer(Sound snd) {
    super("Sound explorer");
    this.snd = snd;
    numFrames = snd.getNumSamples();
    first = 0; last = numFrames - 1;
    setSize(508, 285);
    paint = true;
    setVisible(true);
    setResizable(false);
    this.addListen();
    scale = (last - first) / 500; // The scale we use
    firstTime = false;
  }
  
  /**
   * Adds the mouse, key, and mouseMostion listeners.
   * Used internally
   */
  public void addListen() {
    addKeyListener(this);
    addMouseListener(this);
    addMouseMotionListener(this);
  }
  
  /**
   * Handles the key typed event from
   * Used internally
   */
  public void keyTyped(KeyEvent e) {
  }
  
  /**
   * Handles the key-pressed event
   * Used internally
   */
  public void keyPressed(KeyEvent e) {
    int key = e.getKeyCode();
    if (key == KeyEvent.VK_SPACE) {play(first, last);}
    if (key == KeyEvent.VK_S) {playing = false;}
    if (key == KeyEvent.VK_LEFT)  {
      first--;
      last--;
    }
    if (key == KeyEvent.VK_RIGHT) {
      last++;
      first++;
    }
    if (key == KeyEvent.VK_UP) {
      last += scale;
      first += scale;
    }
    if (key == KeyEvent.VK_DOWN) {
      last -= scale;
      first -= scale;
    }
    
    if (first < 0)
      first = 0;
    if (last > numFrames)
      last = numFrames;
    if (key == KeyEvent.VK_LEFT || key == KeyEvent.VK_RIGHT ||
        key == KeyEvent.VK_UP || key == KeyEvent.VK_DOWN)
      repaint();
  }
  
  /**
   * Handles the key-released event
   * Used internally
   */
  public void keyReleased(KeyEvent e) {
  }
  
  /**
   * Handles the mouse-clicked event
   * Used internally
   */
  public void  mouseClicked(MouseEvent e) {
    int button = e.getButton();
    if (button == MouseEvent.BUTTON1)
      play(((mouseX-3)*scale + first), last);
  }
  
  /**
   * Handles the mouse-entered event
   * Used internally
   */
  public void  mouseEntered(MouseEvent e) {
  }
  
  /**
   * Handles the mouse-exited event
   * Used internally
   */
  public void  mouseExited(MouseEvent e) {
  }
  
  /**
   * Handles the mouse-pressed event
   * Used internally
   */
  public void  mousePressed(MouseEvent e) {
    int button = e.getButton();
    if (button != MouseEvent.BUTTON1) {
      first /= 2;
      last *= 2;
      if (last > numFrames)
        last = numFrames;
      scale = (last - first) / 500; // The scale we use
      repaint();
      paint = true;
    }
  }
  
  /**
   * Handles the mouse-released event
   * Used internally
   */
  public void  mouseReleased(MouseEvent e)  {
    if (dragging) {
      int oF = first;
      int oL = last;
      if (mouseX - dragStart > 0) {
        last = ((mouseX-3)*scale + first);
        first = ((dragStart-3)*scale + first);
      } else {
        last = ((dragStart-3)*scale + first);
        first = ((mouseX-3)*scale + first);
      }
      scale = (last - first) / 500; // The scale we use
      if (scale == 0) {
        first = oF;
        last  = oL;
      }
      if (last > oL)
        last = oL;
      if (first < oF)
        first = oF;
      
      dragging = false;
      paint = true;
      repaint();
    }
  }
  
  /**
   * Handles the mouse-dragged event
   * Used internally
   */
  public void mouseDragged(MouseEvent e) {
    mouseX = e.getX();
    mouseY = e.getY();
    if (!dragging)
      dragStart = mouseX;
    dragging = true;
    paint = true;
    repaint();
  }
  
  /**
   * Handles the mouse-moved event
   * Used internally
   */
  public void mouseMoved(MouseEvent e) {
    mouseX = e.getX();
    mouseY = e.getY();
    paint = false;
    if (mouseY < 231)
      repaint();
  }
  
  /**
   * Paints the window
   * Used internally
   */
  public void paint(Graphics g) {
    scale = (last - first) / 500; // The scale we use
    //scale = 1;
    
    int maxAmp = 0; // Find the biggest amp for y scaling
    for (int j = 0; j < 500; j++) {
      int a = Math.abs(snd.getSample(j*scale + first));
      if (a > maxAmp)
        maxAmp = a;
    }
    
    g.translate(4, 130); // Set the axis to where the actual window starts
    //if (paint || firstTime) {
      g.setColor(Color.white); // BG color is white
      g.fillRect(0,-100, 500, 250); // Draw the BG
      g.setColor(Color.red); // Zero line is red
      g.drawLine(0, 0, 500, 0); // Zero line
      g.setColor(Color.black); // The wave is black
      // Draw the wave 
      for(int i = 0; i < 499; i++) {
        if (i * scale + first > numFrames) {
          System.out.println("Break!");
          break;
        }
        int y1 = snd.getSample(i*scale + first);
        int y2 = snd.getSample(i*scale+scale + first);
        if (maxAmp > 100) {
          if (y1 != 0)
            y1 = y1 * 100 / maxAmp;
          if (y2 != 0)
            y2 = y2 * 100 / maxAmp;
        }
        int x1 = i;
        int x2 = x1 + 1;
        g.drawLine(x1, -y1, x2, -y2);
      }
    //}
    // Display the sample
    g.setColor(Color.white);
    g.fillRect(0, 101, 500, 50);
    g.setColor(Color.black);
    int sample = (mouseX-3)*scale + first;
    if (sample > numFrames - 1)
      sample = numFrames - 1;
    if (sample < 0)
      sample = 0;
    String location = new String("Sample number: " + (sample) + 
                                 " Sample value: " + snd.getSample(sample));
    String blah = new String("Left most sample: " + first + ", right most sample: " + last);
    g.drawString(location, 0, 125);
    g.drawString(blah, 0, 145);
    
    // Draw the highlight when dragging
    Color drag = new Color(0, 200, 150, 100);
    g.setColor(drag);
    if (dragging) {
      if (mouseX - dragStart > 0)
        g.fillRect(dragStart-3, -100, mouseX - dragStart, 201);
 else
        g.fillRect(mouseX-3, -100, dragStart - mouseX, 201);
    }
    paint=true;
  }

  public void play(int first, int last) {
    playing = true;
    // Create the clip
    AudioInputStream ais = snd.makeAIS();
    DataLine.Info info = new DataLine.Info(Clip.class, ais.getFormat());
    Clip clip;
    try {
      clip = (Clip) AudioSystem.getLine(info);
      clip.open(ais);
    } catch (Exception e) {
      System.out.println("Error playing file!");
      return;
    }
    int len = clip.getFrameLength();
    clip.setFramePosition(first);
    clip.start();
    int pos = clip.getFramePosition();
    boolean done = false;
    while (pos < last /* && pos < len */) {
      pos = clip.getFramePosition();
      if (pos > first)
        done = true;
      if (done) {
        if (pos == first)
          break;
      }
    }
    
    clip.close();
  }
}